#include "WiFiWPS.h"
#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>   // Include the Wi-Fi-Multi library
#include "debugWeb.h"

ESP8266WiFiMulti wifiMulti;     // Create an instance of the ESP8266WiFiMulti class, called 'wifiMulti'

bool WiFiWPS::startWPSPBC() {
	// from https://gist.github.com/copa2/fcc718c6549721c210d614a325271389
	Serial.println(G("WPS config start"));
	bool wpsSuccess = WiFi.beginWPSConfig();
	if (wpsSuccess) {
		// Well this means not always success :-/ in case of a timeout we have an empty ssid
		String newSSID = WiFi.SSID();
		if (newSSID.length() > 0) {
			// WPSConfig has already connected in STA mode successfully to the new station. 
			Serial.printf("WPS finished. Connected successfuly to SSID '%s'\n", newSSID.c_str());
		}
		else {
			wpsSuccess = false;
		}
	}
	return wpsSuccess;
}

void WiFiWPS::setupWifi() {

	// Static IP details
	if (eepromUtil.eeprom_read_byte(E_DHCP) == 0) {
		Serial.println(F("WIFI - Static"));
		char buf[5];
		wl_status_t status = WL_IDLE_STATUS;

		eepromUtil.eeprom_read_string(E_SSID, ss.ssid, sizeof(ss.ssid));
		eepromUtil.eeprom_read_string(E_WIFIPASS, ss.wifipass, sizeof(ss.wifipass));
		if ((String)ss.ssid == "") {
			eepromUtil.eeprom_write_byte(E_DHCP, 1);					//Set to DHCP
			EEPROM.commit();
			setupWifi();
			return;
		}

		eepromUtil.eeprom_read_string(E_IPADD, buf, 5, true);
		IPAddress ip(buf[0], buf[1], buf[2], buf[3]);			//Replace with the ip you want to give
		eepromUtil.eeprom_read_string(E_GATEWAY, buf, 5, true);
		IPAddress gateway(buf[0], buf[1], buf[2], buf[3]);		// replace with your router
		eepromUtil.eeprom_read_string(E_MASK, buf, 5, true);
		IPAddress subnet(buf[0], buf[1], buf[2], buf[3]);
		eepromUtil.eeprom_read_string(E_DNS1, buf, 5, true);
		IPAddress dns1(buf[0], buf[1], buf[2], buf[3]);			//replace with router or alternative dns.
		eepromUtil.eeprom_read_string(E_DNS2, buf, 5, true);
		IPAddress dns2(buf[0], buf[1], buf[2], buf[3]);			//replace with router or alternative dns.

		// Set static IP
		WiFi.config(ip, gateway, subnet, dns1, dns2);

		int connectTimeout = 0;
		for (int retry = 0; retry < 5; retry++)
		{
			WiFi.begin(ss.ssid, ss.wifipass); // reading data from EPROM, 

			while ((WiFi.status() == WL_DISCONNECTED) || (WiFi.status() == WL_IDLE_STATUS)) {          // last saved credentials  
				delay(500);
				Serial.print(".");
				connectTimeout++;
				if (connectTimeout == 10)
				{
					Serial.println("Not Connected");
					return;
				}
			}

			status = WiFi.status();
			if ((status == WL_NO_SSID_AVAIL) || (status == WL_CONNECTED))
			{
				break;
			}
		}
	}
	else
	{
		Serial.println(F("WIFI - DHCP"));
		wifiMulti.addAP(SSID_1, PASS_1);   // add Wi-Fi networks you want to connect to
		wifiMulti.addAP(SSID_2, PASS_2);   // add Wi-Fi networks you want to connect to

		Serial.println("Connecting ...");
		while (wifiMulti.run(20000) != WL_CONNECTED) { // Wait for the Wi-Fi to connect: scan for Wi-Fi networks, and connect to the strongest of the networks above
			delay(1000);
			Serial.print('.');
		}
	}

	Serial.println('\n');
	Serial.print("Connected to ");
	Serial.println(WiFi.SSID());              // Tell us what network we're connected to
	Serial.print("IP address:\t");
	Serial.println(WiFi.localIP());           // Send the IP address of the ESP8266 to the computer

	//int connectTimeout = 0;
	//wl_status_t status = WL_IDLE_STATUS;
	//String wifissid = WiFi.SSID();
	//String wifipsk = WiFi.psk();

	//Serial.println(F("Startup WiFi.."));
	//Serial.printf("Try connecting to WiFi with SSID '%s', password '%s'\n", wifissid.c_str(), wifipsk.c_str());
	//WiFi.disconnect();

	//WiFi.mode(WIFI_STA);

	//if (wifissid != "") {
	//	for (int retry = 0; retry < 5; retry++)
	//	{
	//		WiFi.begin(wifissid.c_str(), wifipsk.c_str()); // reading data from EPROM, 

	//		while ((WiFi.status() == WL_DISCONNECTED) || (WiFi.status() == WL_IDLE_STATUS)) {          // last saved credentials  
	//			delay(500);
	//			Serial.print(".");
	//			connectTimeout++;
	//			if (connectTimeout == 10)
	//			{
	//				Serial.println("Not Connected");
	//				return;
	//			}
	//		}

	//		status = WiFi.status();
	//		if ((status == WL_NO_SSID_AVAIL) || (status == WL_CONNECTED))
	//		{
	//			break;
	//		}
	//	}
	//}

	//if (status == WL_CONNECTED) {
	//	Serial.printf("Connected successful to SSID '%s'\n", WiFi.SSID().c_str());
	//}
	//else {
	//	// WPS button I/O setup
	//	Serial.println(F("Start WPS"));

	//	if (!startWPSPBC()) {
	//		Serial.println(F("Failed to connect with WPS :-("));
	//		delay(3000);
	//	}
	//	else if (WiFi.isConnected())
	//		return;
	//	else
	//	{
	//		WiFi.begin(WiFi.SSID().c_str(), WiFi.psk().c_str()); // reading data from EPROM, 
	//		connectTimeout = 0;
	//		while (WiFi.status() == WL_DISCONNECTED) {          // last saved credentials
	//			delay(1000);
	//			Serial.print(".2");								// show wait for connect to AP
	//			connectTimeout++;
	//			if (connectTimeout == 30)
	//				ESP.restart();
	//		}
	//	}
	//}
}

void WiFiWPS::resetSettings() {
	Serial.println(G("settings invalidated"));
	Serial.println(G("THIS MAY CAUSE AP NOT TO START UP PROPERLY. YOU NEED TO COMMENT IT OUT AFTER ERASING THE DATA."));
	WiFi.disconnect(true);
}

WiFiWPS wifiWPS;
